/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef __H_LINKED_LIST__
#define __H_LINKED_LIST__

#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>
#include "defs.h"

/** @file
 * A LinkedList represents a linked list. A linked list is an efficient data
 * structure. It is not posible to index a linked list.
 */

BEGIN_DECLS

/**
 * Represents an object in a LinkedList.
 */
typedef struct LinkedListNode LinkedListNode;

/**
 * A linked list is a data stucture that holds information in a non-idexible
 * way. It is good when the objects in the list in verry mutable, but has a
 * linear search time.
 */
typedef struct LinkedList LinkedList;

/**
 * Create a new linked list.
 * 
 * @param free_func the function to be used when freeing a linked list,
 * if it is NULL the free func will be set to the standard free() function.
 */
LinkedList * linked_list_new (void (* free_func)(void *));

/**
 * frees a linked list.
 * 
 * @param self the linked list to free.
 * @param free_data default:true, wether or not to free the data in the list
 *   as well as the list.
 */
void linked_list_free (LinkedList * self, bool free_data);

/**
 * Add an item to a linked list.
 *
 * @param self the list to add an itme to.
 * @param data the data to be stored.
 */
void linked_list_add (LinkedList * self, void * data);

/**
 * Iterate to the next item. This function does not return anything.
 * 
 * @param self the list to iterate over.
 */
void linked_list_next (LinkedList * self);

/**
 * Iterate to the provious item. This function does not return anything.
 * 
 * @param self the list to iterate over.
 */
void linked_list_pev (LinkedList * self);

/**
 * Get the current item.
 */
void * linked_list_get_current (LinkedList * self);

/**
 * Get the next item.
 */
void * linked_list_get_next (LinkedList * self);

/**
 * Get previous item.
 */
void * linked_list_get_prev (LinkedList * self);

/**
 * Wind the list to the head (start).
 */
void linked_list_head (LinkedList * self);

/**
 * Wind the the list to the tail (end).
 */
void linked_list_tail (LinkedList * self);

/**
 * Get the length of the list.
 *
 * @warning the value may be wrong.
 */
size_t linked_list_len (LinkedList * self);

/**
 * Romoves the cerrunt object useing the free function provided when
 * creating the list in linked_list_new ().
 */
void linked_list_remove_current (LinkedList * self);

END_DECLS

#endif /* __H_LINKED_LIST__ */
