/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/** @file */




#ifndef __H_BASE_OBJECT__
#define __H_BASE_OBJECT__


/** @file
 * An SObject represents the base of the Super Simple Type System.
 * 
 * Almost all other data structures in this library are children to this object
 * type.
 *
 * All objects that inherent from SObject get the ability to have
 * reference counting of objects via built in functions.
 */

/* ---------------------------------------------------
 * This is just a test of creating a small typesystem.
 * It will include refrence counting.
 * ---------------------------------------------------
 */

#include "Func.h"

BEGIN_DECLS

typedef struct _SObjectClass SObjectClass;

typedef struct _SObject SObject;


/**
 *
 */
typedef void (* FreeMethod)(SObject *);

/**
 *
 */
typedef char * (* ToStringFunc)(SObject *);

/**
 * 
 */
typedef void (* MethodFunc)(SObject *);

/**
 *
 */
typedef int (* MethodFuncInt)(SObject *);


#define S_OBJECT(k) (SObject *)k

/**
 * The class holds all the "virtual" functions, also knows  as methods
 * that are to be used with the object.
 * 
 */
struct _SObjectClass {
  MethodFunc initialize;
  MethodFunc deinitialize;
  FreeMethod free;
  MethodFuncInt ref;
  MethodFuncInt unref;
  MethodFuncInt get_refcount;
  ToStringFunc to_string;
};

/**
 * The instance holds the data that is associated with the object.
 * it also holds a pointer to the class of the object.
 * 
 */
struct _SObject {
  char * name;
  SObjectClass * base_class;
  unsigned int refcount;
};


/* -----------------
 * Helper functions...
 * -----------------
 */
/** This function is used to set the method to initialize a new instance of an
 * object.
 * 
 * 
 */
void s_object_set_initialize_method (SObject * self, MethodFunc method);

/**
 * This function is used to set the method to deinitialize an object.
 * 
 * set it to a method that deinitialize your object.
 */
void s_object_set_deinitialize_method (SObject * self, MethodFunc method);

/**
 * This function is used to set the ref method.
 * 
 * DO NOT USE THIS UNLESS YOU KNOW WHAT YOU ARE DOING.
 */
void s_object_set_ref_method (SObject * self, MethodFuncInt method);

/**
 * This function is used to set the unref method.
 * 
 * DO NOT USE THIS UNLESS YOU KNOW WHAT YOU ARE DOING.
 */
void s_object_set_unref_method (SObject * self, MethodFuncInt method);

/**!
 * This function is used to set the get_refcount method.
 * 
 * DO NOT USE THIS UNLESS YOU KNOW WHAT YOU ARE DOING.
 */
void s_object_set_get_refcount_method (SObject * self,  MethodFuncInt method);

/**!
 * This function is used to set the to_string method.
 */
void s_object_set_to_string_method (SObject * self, ToStringFunc method);

/**
 *
 */
void s_object_set_free_method (SObject * self, MethodFunc method);


/* concrete functions are defined in the C file.
 */

/* ----------------------
 * Base object functions.
 * ----------------------
 */

/** @brief
 * Initializes an SObject
 * 
 * @param self The object to initialize.
 * 
 * This function initializes an instance of the SObject, it also sets
 * the methods to be used with the object and sets the reference count to one.
 */
void s_object_initialize (SObject * self);

/** @brief
 * This function creates a new base object.
 * 
 * 
 * 
 * @return a new SObject
 */
SObject * s_object_new ();

/**
 * This function desensitizes/frees an object even if it is still referenced.
 * This is usually a bad idea, use s_object_unref () instead.
 */
void s_object_free (SObject * self);

/**
 * This function gets the class (which hold the object methods).
 */
void * s_object_get_class (SObject * self);

/**
 * This function sets the instance class of an object.
 */
void s_object_set_class (SObject * self, SObjectClass * klass);

/**
 * This function is used to decrese the reference count of an object.
 * When an object reaches zero, it will deinitialize the object using the
 * objects deinitialize method.
 * 
 * It returns the current (after change) reference count.
 */
int s_object_unref (SObject * self);

/**
 * This function is used to increase the reference count of an object.
 *
 * Returns the current (after change) reference count.
 */
int s_object_ref (SObject * self);

/**
 * This function returns the current reference count without changing it.
 */
int s_object_get_refcount (SObject * self);

/**
 * This function returns a textual (string) that represents the object.
 * The method can be set using s_object_set_to_string_method.
 *
 * Note: The string that is returned must be freed.
 */
char * s_object_to_string (SObject * self);

END_DECLS

#endif
